/*
 Wireless Gameboy Link (Packet based)
 Version: 1.0
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 8/12/2018
 Last Modified: 9/12/2018
 
 Uses the nRF24 wireless module.
 Not compatible with existing games as it's packet based. 
 You could use this for your own games or potentially patch  existing games.
 
*/

// ATtiny441/841 Pin map
//
//                     +-\/-+
//               VCC  1|o   |14  GND
//   GB Data out PB0  2|    |13  PA0 GB CLK
//               PB1  3|    |12  PA1 GB Data in
//         Reset PB3  4|    |11  PA2 LED
//               PB2  5|    |10  PA3 nRF24L01 CE
//  nRF24L01 CSN PA7  6|    |9   PA4 nRF24L01 SCK
//   nRF24L01 DI PA6  7|    |8   PA5 nRF24L01 DO
//                     +----+

#define F_CPU 8000000

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <math.h>
#include <util/delay.h>
#include "setup.c"
#include "mirf.c"

uint8_t nrfDataIn[6];
uint8_t nrfDataOut[6];
uint8_t volatile transmitData = 0;

// Read bits when clock is low
void read_and_write_bytes(uint8_t count) {
	for (uint8_t c = 0; c < count; c++) {
		nrfDataOut[c] = 0; // Clear data out
		
		for (int8_t x = 7; x >= 0; x--) {
			while (PINA & (1<<GB_CLK)); // Wait until clock is low
			asm volatile("nop");
			
			// Write bits to GB
			if (nrfDataIn[c] & (1<<x)) {
				PORTB |= (1<<DATA_OUT);
			}
			else {
				PORTB &= ~(1<<DATA_OUT);
			}
			
			while (!(PINA & (1<<GB_CLK))); // Wait until clock is high
			asm volatile("nop");
			
			// Read bits from GB
			if (PINA & (1<<DATA_IN)) {
				nrfDataOut[c] |= (1<<x);
			}
		}
	}
}


int main (void) {
	setup();
	
	RX_POWERUP;
	_delay_ms(3);
	mirf_flush_rx_tx(); // Flush TX/RX
	mirf_CE_hi; // Start listening
	
	// Enable pin change interrupt on PA0
	PCMSK0 |= (1<<PCINT0);
	GIMSK |= (1<<PCIE0);
	
	while(1) {
		if (mirf_status() & (1<<RX_DR)) {
			PORTA |= (1<<LED);
			mirf_CE_lo; // Stop listening
			
			mirf_CSN_lo; // Pull down chip select
			spi_transfer(R_RX_PAYLOAD); // Send cmd to read rx payload
			spi_read_data(nrfDataIn, mirf_PAYLOAD); // Read payload
			
			mirf_CSN_hi; // Pull up chip select
			mirf_config_register(STATUS,(1<<RX_DR)); // Reset status register
			
			mirf_flush_rx_tx(); // Flush TX/RX
			mirf_CE_hi; // Start listening
			PORTA &= ~(1<<LED);
		}
		
		if (transmitData == 1) { // Transmit a packet
			mirf_CE_lo; // Stop listening
			
			transmitData = 0;
			mirf_transmit_data();
			
			RX_POWERUP;
			_delay_ms(3);
			mirf_flush_rx_tx(); // Flush TX/RX
			mirf_CE_hi; // Start listening
		}
		
		_delay_us(250);
	}
	
	return 0;
}


ISR(PCINT0_vect) {
	if (!(PINA & (1<<GB_CLK))) {
		// Read bytes to send while writing bytes last received
		read_and_write_bytes(6);
		
		transmitData = 1;
		_delay_us(50); // Wait a little so we don't re-trigger the interrupt
	}
}
